package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewCatalog
import dev.fyloz.colorrecipesexplorer.config.annotations.RequireDatabase
import dev.fyloz.colorrecipesexplorer.dtos.CompanyDto
import dev.fyloz.colorrecipesexplorer.logic.CompanyLogic
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

private const val COMPANY_CONTROLLER_PATH = "api/company"

@RestController
@RequestMapping(COMPANY_CONTROLLER_PATH)
@RequireDatabase
@PreAuthorizeViewCatalog
class CompanyController(private val companyLogic: CompanyLogic) {
    @GetMapping
    fun getAll() =
        ok(companyLogic.getAll())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(companyLogic.getById(id))

    @PostMapping
    @PreAuthorize("hasAuthority('EDIT_COMPANIES')")
    fun save(@Valid @RequestBody company: CompanyDto) =
        created<CompanyDto>(COMPANY_CONTROLLER_PATH) {
            companyLogic.save(company)
        }

    @PutMapping
    @PreAuthorize("hasAuthority('EDIT_COMPANIES')")
    fun update(@Valid @RequestBody company: CompanyDto) =
        noContent {
            companyLogic.update(company)
        }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_COMPANIES')")
    fun deleteById(@PathVariable id: Long) =
        noContent {
            companyLogic.deleteById(id)
        }
}
