package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewCatalog
import dev.fyloz.colorrecipesexplorer.logic.MaterialTypeLogic
import dev.fyloz.colorrecipesexplorer.model.MaterialType
import dev.fyloz.colorrecipesexplorer.model.MaterialTypeSaveDto
import dev.fyloz.colorrecipesexplorer.model.MaterialTypeUpdateDto
import org.springframework.context.annotation.Profile
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

private const val MATERIAL_TYPE_CONTROLLER_PATH = "api/materialtype"

@RestController
@RequestMapping(MATERIAL_TYPE_CONTROLLER_PATH)
@Profile("!emergency")
@PreAuthorizeViewCatalog
class MaterialTypeController(private val materialTypeLogic: MaterialTypeLogic) {
    @GetMapping
    fun getAll() =
      ok(materialTypeLogic.getAllForOutput())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
      ok(materialTypeLogic.getByIdForOutput(id))

    @PostMapping
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun save(@Valid @RequestBody materialType: MaterialTypeSaveDto) =
      created<MaterialType>(MATERIAL_TYPE_CONTROLLER_PATH) {
          materialTypeLogic.save(materialType)
      }

    @PutMapping
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun update(@Valid @RequestBody materialType: MaterialTypeUpdateDto) =
      noContent {
          materialTypeLogic.update(materialType)
      }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun deleteById(@PathVariable id: Long) =
      noContent {
          materialTypeLogic.deleteById(id)
      }
}

